"""
Module for common terms definition.
"""

# Nefertem version
NEFERTEM_VERSION: str = "2.0.0"

# Data readers type
FILE_READER: str = "file_reader"
NATIVE_READER: str = "native_readerr"

# Generics
DUMMY: str = "_dummy"
